<?php
	// CONFIG
	$param['cache_duration']= 3600 * 24 * 10;		// 10 days util client cache expires
	$param['debug'] = false;							// Enable this option if you need debuging info
	$param['use_disk_cache']= true;					// If you enable this option gzip files will be cached on disk.
	$param['use_gzip']= true;						// Enable gzip compression
	// END CONFIG
	
	$compressor= new Compressor($param);
	
	class Compressor{
		/*var $path= ""; // path of the script
		var $datas= ""; // datas to be sent to user
	*/
		
		function compressor($param)
		{
			$this->__construct($param);
		}
		
		function __construct($param)
		{
			$this->param= $param;
			$this->script_list="";
			$this->path= dirname(__FILE__)."/";
			$this->full_cache_file= $this->path."edit_area_full.js";
			$this->gzip_cache_file= $this->path."edit_area_full.gz";
			
			
			$this->check_gzip_use();
			$this->send_headers();
			$this->check_cache();
			$this->load_files();
			$this->send_datas();
		}
		
		function send_headers()
		{
			header("Content-type: text/javascript; charset: UTF-8");
			header("Vary: Accept-Encoding"); // Handle proxies
			header(sprintf("Expires: %s GMT", gmdate("D, d M Y H:i:s", time() + $this->param['cache_duration'])) );
			if($this->use_gzip)
				header("Content-Encoding: gzip");
		}
		
		function check_gzip_use()
		{
			$encodings = array();		
			if (isset($_SERVER['HTTP_ACCEPT_ENCODING']))
				$encodings = explode(',', strtolower(preg_replace("/\s+/", "", $_SERVER['HTTP_ACCEPT_ENCODING'])));
		
			// Check for gzip header or northon internet securities
			if ($this->param['use_gzip'] && (in_array('gzip', $encodings) || isset($_SERVER['---------------'])) && function_exists('ob_gzhandler') && !ini_get('zlib.output_compression')) {
				$this->use_gzip=true;
				$this->cache_file=$this->gzip_cache_file;
			}else{
				$this->use_gzip=false;
				$this->cache_file=$this->full_cache_file;
			}
		}
		
		function check_cache()
		{
			// Only gzip the contents if clients and server support it
			if (file_exists($this->cache_file)) {
				// check if cache file must be updated
				$cache_date=0;				
				if ($dir = opendir($this->path)) {
					while (($file = readdir($dir)) !== false) {
						if(is_file($this->path.$file) && $file!="." && $file!="..")
							$cache_date= max($cache_date, filemtime($this->path.$file));
					}
					closedir($dir);
				}

				if(filemtime($this->cache_file) >= $cache_date){
					// if cache file is up to date
					$last_modified = gmdate("D, d M Y H:i:s",filemtime($this->cache_file))." GMT";
					if (isset($_SERVER["HTTP_IF_MODIFIED_SINCE"]) && strcasecmp($_SERVER["HTTP_IF_MODIFIED_SINCE"], $last_modified) === 0)
					{
						header("HTTP/1.1 304 Not Modified");
						header("Last-modified: ".$last_modified);
						header("Cache-Control: Public"); // Tells HTTP 1.1 clients to cache
						header("Pragma:"); // Tells HTTP 1.0 clients to cache
					}
					else
					{
						header("Last-modified: ".$last_modified);
						header("Cache-Control: Public"); // Tells HTTP 1.1 clients to cache
						header("Pragma:"); // Tells HTTP 1.0 clients to cache
						header('Content-Length: '.filesize($this->cache_file));
						echo file_get_contents($this->cache_file);
					}				
					die;
				}
			}
			return false;
		}
		
		function load_files()
		{
			$loader= $this->get_content("edit_area_loader.js")."\n";
			
			// get the list of other files to load
			/*create_function(
	           				'$matches',
	           				'global $compressor;
	           				 $this->script_list.=$matches[2];
	           				 return $matches[1]."();";'
	         			)*/
	        
			$loader= preg_replace("/(this\.scripts_to_load= new Array)\(([^\)]*)\);/e"
						, "\$this->replace_scripts('script_list', '\\1', '\\2')"
						, $loader);
		
			$loader= preg_replace("/(this\.sub_scripts_to_load= new Array)\(([^\)]*)\);/e"
						, "\$this->replace_scripts('sub_script_list', '\\1', '\\2')"
						, $loader);
			
			
			$this->datas= $loader;
			
			// load other scripts needed for the loader
			preg_match_all('/"([^"]*)"/', $this->script_list, $match);
			foreach($match[1] as $key => $value)
				$this->datas.= $this->get_content($value.".js")."\n";
		
			$this->compress_javascript($this->datas);
			//$this->datas= preg_replace('/(( |\t|\r)*\n( |\t)*)+/s', "", $this->datas);
			$this->datas.="\n";
			
			// load sub scripts
			$sub_scripts="";
			preg_match_all('/"([^"]*)"/', $this->sub_script_list, $match);			
			foreach($match[1] as $key => $value)
				$sub_scripts.= $this->get_javascript_content($value.".js");
				
			$this->datas.= sprintf("editAreaLoader.iframe_script= \"<script language='Javascript' type='text/javascript'>%s</script>\";\n", $sub_scripts);
			
			// load the template
			$this->datas.= sprintf("editAreaLoader.template= \"%s\";\n", $this->get_javascript_content("template.html"));
			// load the css
			$this->datas.= sprintf("editAreaLoader.iframe_css= \"<style>%s</style>\";\n", $this->get_javascript_content("edit_area.css"));
		//	$this->datas= "function editArea(){};editArea.prototype.loader= function(){alert('bouhbouh');} var a= new editArea();a.loader();";		
		}
		
		function send_datas()
		{
			
			
			if($this->param['debug']){
				$header=sprintf("/* USE PHP COMPRESSION\n");
				if($this->use_gzip){
					$gzip_datas=  gzencode($this->datas, 9, FORCE_GZIP);				
					$ratio = round(100 - strlen($gzip_datas) / strlen($this->datas) * 100.0);			
					$header.=sprintf("javascript size: base= %s, gzip= %s, reduced by %s%%\n", strlen($this->datas), strlen($gzip_datas), $ratio);				
				}else{
					$header.=sprintf("javascript size: %s\n", strlen($this->datas));
				}
				$header.=sprintf("%s\n", implode("\n", $this->infos));
				$header.=sprintf("*/\n");
				$this->datas= $header.$this->datas;	
			}
			$mtime= time(); // ensure that the 2 disk files will have the same update time
			// generate gzip file and cahce it if using disk cache
			if($this->use_gzip){
				$this->gzip_datas= gzencode($this->datas, 9, FORCE_GZIP);
				if($this->param['use_disk_cache'])
					$this->file_put_contents($this->gzip_cache_file, $this->gzip_datas, $mtime);
			}
			
			// generate full js file and cache it if using disk cache			
			if($this->param['use_disk_cache'])
				$this->file_put_contents($this->full_cache_file, $this->datas, $mtime);
		
			// generate output
			if($this->use_gzip)
				echo $this->gzip_datas;
			else
				echo $this->datas;
				
			die;
		}
				
		
		function get_content($name)
		{
			$name=preg_replace("/\\|\//i", "", $name); // Remove any / or \
			$file= $this->path.$name;
			if(file_exists($file)){
				$this->infos[]=sprintf("'%s' loaded", $name);
				/*$fd = fopen($file, 'rb');
				$content = fread($fd, filesize($file));
				fclose($fd);
				return $content;*/
				return $this->file_get_contents($file);
			}else{
				$this->infos[]=sprintf("'%s' not loaded", $name);
				return "";
			}
		}
		
		function get_javascript_content($name)
		{
			$val=$this->get_content($name);
			// remove comments
		
			// js: /("(?:\\"|[^"])*(?:"|$))|('(?:\\'|[^'])*(?:'|$))|(?:\/\/(?:.|\r|\t)*?(?:\n|$))|(?:\/\*(?:.|\n|\r|\t)*?(?:\*\/|$))/gi
			//$val= preg_replace('/("(?:\\"|[^"])*(?:"|$))|(\'(?:\\\'|[^\'])*(?:\'|$))|(?:\/\/(?:.|\r|\t)*?(?:\n|$))|(?:\/\*(?:.|\n|\r|\t)*?(?:\*\/|$))/ms', "$1$2", $val);
			
			// remove all comments
			//$val= preg_replace('/(\"(?:\\\"|[^\"])*(?:\"|$))|' . "(\'(?:\\\'|[^\'])*(?:\'|$))|(?:\/\/(?:.|\r|\t)*?(\n|$))|(?:\/\*(?:.|\n|\r|\t)*?(?:\*\/|$))/ms", "$1$2$3", $val);
			$this->compress_javascript($val);
			$pattern= array('/"/', '/\\\n/'	, '/\\\r/'	, "/(\r?\n)/");
			$replace= array('\\"', '\\\\\\n', '\\\\\\r'	, '\\\n"$1+"');
			$val= preg_replace($pattern, $replace, $val);
			return $val;
		}
		
		function compress_javascript(&$code)
		{
			// remove all comments
			$code= preg_replace('/(\"(?:\\\"|[^\"])*(?:\"|$))|' . "(\'(?:\\\'|[^\'])*(?:\'|$))|(?:\/\/(?:.|\r|\t)*?(\n|$))|(?:\/\*(?:.|\n|\r|\t)*?(?:\*\/|$))/s", "$1$2$3", $code);
			// remove line return, empty line and tabulation
			//$code= preg_replace('/([^)e}])(( |\t|\r)*\n( |\t)*)+/s', "$1", $code);
			$code= preg_replace('/(( |\t|\r)*\n( |\t)*)+/s', "\n", $code);
		}
		
		function replace_scripts($var, $param1, $param2)
		{
			$this->$var=stripslashes($param2);
	        return $param1."();";
		}

		/* for php version that have not thoses functions */
		function file_get_contents($file)
		{
			$fd = fopen($file, 'rb');
			$content = fread($fd, filesize($file));
			fclose($fd);
			return $content;				
		}
		
		function file_put_contents($file, &$content, $mtime=-1)
		{
			if($mtime==-1)
				$mtime=time();
			$fp = @fopen($file, "wb");
			if ($fp) {
				fwrite($fp, $content);
				fclose($fp);
				touch($file, $mtime);
				return true;
			}
			return false;
		}
	}	
?>
